#include "sysctrl.h"
#include "reset.h"
#include "ddl.h"
#include "hc32l19x.h"


static void __sysctrl_unlock (void)
{
    M0P_SYSCTRL->SYSCTRL2 = 0x5A5A;
    M0P_SYSCTRL->SYSCTRL2 = 0xA5A5;
}

static void __flash_unlock (void)
{
    M0P_FLASH->BYPASS = 0x5A5A;
    M0P_FLASH->BYPASS = 0xA5A5;
}




void sysclk_init (void)
{

    uint8_t hclk_mhz = 0;

    /* ʹFLASHʱ */
    Sysctrl_SetPeripheralGate(SysctrlPeripheralFlash, TRUE);

    /*  PLL_CR.REFSEL Ϊ 3ѡPLLʱԴΪRCHPLLʱԴ */
    M0P_SYSCTRL->PLL_CR_f.REFSEL = 3;

    /* RCHʱƵУ׼ֵ */
    Sysctrl_SetRCHTrim(SysctrlRchFreq4MHz);

    __sysctrl_unlock();
    /* ڲʱRCHʹ */
    M0P_SYSCTRL->SYSCTRL0_f.RCH_EN = 1;

    /* ȴ RCH_CR. Stable ־Ϊ 1 RCHȶʱ */
    while (M0P_SYSCTRL->RCH_CR_f.STABLE != 1);

    /*  PLL Ƶʣ PLL_CR.DIVN  PLL_CR.FOSC */
    M0P_SYSCTRL->PLL_CR_f.DIVN = 12;

    /*  RCH Ƶʣ PLL_CR.FRSEL */
    /* PLL Ƶѡ񣬸PLLʱӵƵʽ 4M~6M */
    M0P_SYSCTRL->PLL_CR_f.FRSEL = 0;

    /* PLL ƵʷΧѡ񣬸PLLʱƵʽã36M~48M */
    M0P_SYSCTRL->PLL_CR_f.FOSC = 4; /* 4~7ǿԵ */

    /* PLLȶʱѡ */
    /*  PLL_CR.Startup Ϊ 7ѡ PLL ȶʱ, 16384PLL */
    M0P_SYSCTRL->PLL_CR_f.STARTUP = 7;

    __sysctrl_unlock();
    /*  SYSCTRL0.PLL_EN Ϊ 1ʹ PLL 񵴵· */
    M0P_SYSCTRL->SYSCTRL0_f.PLL_EN = 1;

    /* ѯȴ PLL_CR. Stable ־Ϊ 1 PLL ȶʱ */
    while (M0P_SYSCTRL->PLL_CR_f.STABLE != 1);

    __flash_unlock();

    /* FLASH   24~48MHzΪ2 */
    M0P_FLASH->CR_f.WAIT = 1;

    __sysctrl_unlock();
    /*  SYSCTRL0. CLKSW Ϊ 4ϵͳʱлΪ PLL */
    M0P_SYSCTRL->SYSCTRL0_f.CLKSW = 4;

 
    /* ʱӷƵ */
    /* HCLKƵϵHCLK = SYSCLK / (2 ^ hclk_div) */
    Sysctrl_SetHCLKDiv(SysctrlHclkDiv1);
    hclk_mhz = 48;  /* 48MHz */

    /* PCLKƵϵPCLK = HCLK / (2 ^ pclk_div) */
    Sysctrl_SetPCLKDiv(SysctrlPclkDiv2);


    /* 㹫ʽ TNVS = 8*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TNVS = hclk_mhz * 4;

    /* 㹫ʽ TPGS = 5.75*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TPGS = hclk_mhz * 5.75;

    /* 㹫ʽ 㹫ʽ TPROG = 6.75*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TPROG = hclk_mhz * 6.75;

    /* 㹫ʽ TSERASE = 4500*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TSERASE = hclk_mhz * 4500;

    /* 㹫ʽ TMERASE = 35000*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TMERASE = hclk_mhz * 35000;

    /* 㹫ʽ TPRCV = 6*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TPRCV = hclk_mhz * 6;

    /* 㹫ʽ TSRCV = 60*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TSRCV = hclk_mhz * 60;

    /* 㹫ʽ TMRCV = 250*HCLK HCLKĵλΪMHz */
    __flash_unlock();
    M0P_FLASH->TMRCV = hclk_mhz * 250;

    SystemCoreClock = 48000000;

}










