/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief GPIO
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */



#include "stdio.h"

#include "board_gpio.h"
#include "gpio.h"
#include "spi.h"


/* GPIOųʼ  */
void mcu_gpio_init (void)
{

    stc_gpio_cfg_t GpioInitStruct;
    DDL_ZERO_STRUCT(GpioInitStruct);

    Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);

    /* RST  PIOC_10 */
    GpioInitStruct.enDir = GpioDirOut;           /*  */
    GpioInitStruct.enPu  = GpioPuEnable;         /*  */
    GpioInitStruct.enPd  = GpioPdDisable;        /*  */
    Gpio_Init(GpioPortC, GpioPin10, &GpioInitStruct);
    Gpio_WriteOutputIO(GpioPortC, GpioPin10, 1); /* Ĭߵƽ */

    /* BUSY  PIOC_11 */
    GpioInitStruct.enDir = GpioDirIn;       /*  */
    GpioInitStruct.enPu  = GpioPuDisable;   /*  */
    GpioInitStruct.enPd  = GpioPdEnable;    /*    */
    Gpio_Init(GpioPortC, GpioPin11, &GpioInitStruct);

    /* DIO1  PIOC_12 */
    GpioInitStruct.enDir = GpioDirIn;       /*  */
    GpioInitStruct.enPu  = GpioPuDisable;   /*  */
    GpioInitStruct.enPd  = GpioPdEnable;    /*    */
    Gpio_Init(GpioPortC, GpioPin12, &GpioInitStruct);


    /* RXEN  PIOF_5 */
    GpioInitStruct.enDir = GpioDirOut;          /*  */
    GpioInitStruct.enPu  = GpioPuEnable;        /*  */
    GpioInitStruct.enPd  = GpioPdDisable;       /*  */
    Gpio_Init(GpioPortF, GpioPin5, &GpioInitStruct);
    Gpio_WriteOutputIO(GpioPortF, GpioPin5, 1); /* Ĭߵƽ */
    
    /* TXEN  PIOF_4 */
    GpioInitStruct.enDir = GpioDirOut;          /*  */
    GpioInitStruct.enPu  = GpioPuEnable;        /*  */
    GpioInitStruct.enPd  = GpioPdDisable;       /*  */
    Gpio_Init(GpioPortF, GpioPin4, &GpioInitStruct);
    Gpio_WriteOutputIO(GpioPortF, GpioPin4, 1); /* Ĭߵƽ */


    /* LED_ACK   ɫ  PIOC_4 */
    GpioInitStruct.enDir = GpioDirOut;          /*  */
    GpioInitStruct.enPu  = GpioPuEnable;        /*  */
    GpioInitStruct.enPd  = GpioPdDisable;       /*  */
    Gpio_Init(GpioPortC, GpioPin4, &GpioInitStruct);
    Gpio_WriteOutputIO(GpioPortC, GpioPin4, 1); /* Ĭߵƽ */


    /* LED_STATE  ɫ  PIOA_3 */
    GpioInitStruct.enDir = GpioDirOut;          /*  */
    GpioInitStruct.enPu  = GpioPuEnable;        /*  */
    GpioInitStruct.enPd  = GpioPdDisable;       /*  */
    Gpio_Init(GpioPortA, GpioPin3, &GpioInitStruct);
    Gpio_WriteOutputIO(GpioPortA, GpioPin3, 1); /* Ĭߵƽ */

}



/*
 * SX_NRESET λŵƽáýӿڱṩ
 * val=0ʾΪ͵ƽval =1ʾΪߵƽ
 */ 
void mcu_gpio_rst_pin_set (uint8_t val)
{
    if (val == 1) {
        Gpio_WriteOutputIO(GpioPortC, GpioPin10, 1);
    } else {
        Gpio_WriteOutputIO(GpioPortC, GpioPin10, 0);
    }
}


/*
 * SPI_NSS Ƭѡŵƽáýӿڱṩ
 * val=0ʾΪ͵ƽval =1ʾΪߵƽ
 */ 
void mcu_gpio_sel_pin_set (uint8_t val)
{
    if (val == 1) {
        Gpio_WriteOutputIO(GpioPortA, GpioPin15, 1);
        Spi_SetCS(M0P_SPI0, 1);
    } else {
        Gpio_WriteOutputIO(GpioPortA, GpioPin15, 0);
        Spi_SetCS(M0P_SPI0, 0);
    }
}

/*
 * Ƶطŵƽáýӿڿɸģѡṩ
 * ΪZSL420ZSL421ZM1268S ZM68Sģ飬ṩ˽ӿ
 * val=0ʾΪ͵ƽval =1ʾΪߵƽ
 * ΪZSL64ϵģ飬ֱô˽ӿΪNULL߽ӿʵΪ
 */
void mcu_gpio_txen_pin_set (uint8_t val)
{
    if (val == 1) {
        Gpio_WriteOutputIO(GpioPortF, GpioPin4, 1);
    } else {
        Gpio_WriteOutputIO(GpioPortF, GpioPin4, 0);
    }
}

/*
 * VCTL_1(RXEN) Ƶؽտŵƽáýӿڱṩ
 * val=0ʾΪ͵ƽval =1ʾΪߵƽ
 */
void mcu_gpio_rxen_pin_set (uint8_t val)
{
    if (val == 1) {
        Gpio_WriteOutputIO(GpioPortF, GpioPin5, 1);
    } else {
        Gpio_WriteOutputIO(GpioPortF, GpioPin5, 0);
    }
}

/*
 * BUSY æŵƽȡýӿڱṩ
 * 0ʾΪ͵ƽ1ʾΪߵƽ
 */
uint8_t mcu_gpio_busy_pin_read (void)
{ 
    return (Gpio_GetInputIO(GpioPortC, GpioPin11));
}


/* GPIOŵƽת */
void mcu_gpio_pin_toggle (en_gpio_port_t enPort, en_gpio_pin_t enPin)
{
    if (Gpio_ReadOutputIO(enPort, enPin) == 0) {
        Gpio_WriteOutputIO(enPort, enPin, 1);
    } else {
        Gpio_WriteOutputIO(enPort, enPin, 0);
    }
}

 












