/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief UART
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */

#include "stdio.h"
#include "stdlib.h"
#include "string.h"

#include "ddl.h"
#include "gpio.h"
#include "lpuart.h"
#include "sysctrl.h"
#include "reset.h"

#include "board_uart.h"






#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
  
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART */
  mcu_uart_byte_send((uint8_t)ch);

  return ch;
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}

#endif



static uart_t    __g_uart_dev;
static uint8_t   __g_uart_init = 0;


static void __delay_ms (uint32_t ms)
{
    uint32_t i = 0;
    uint32_t j = 0;

    while (i < ms) {
        i++;
        for(j = 0; j < 4369; j++);
    }
}

/* 崮ڳʼ */
void mcu_uart_init (uint32_t baud_rate, uint16_t buf_size, uart_rx_callback_t rx_callback)
{
    uint8_t  *p_buf;

    if (__g_uart_init) {
        return;
    }

    stc_gpio_cfg_t stcGpioCfg;

    DDL_ZERO_STRUCT(stcGpioCfg);

    Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);

    stcGpioCfg.enDir =  GpioDirOut;
    Gpio_Init(GpioPortB,GpioPin12,&stcGpioCfg);
    Gpio_SetAfMode(GpioPortB,GpioPin12,GpioAf3); /* PB12ΪLPUART0_TX */


    stcGpioCfg.enDir =  GpioDirIn;
    Gpio_Init(GpioPortB,GpioPin11,&stcGpioCfg);
    Gpio_SetAfMode(GpioPortB,GpioPin11,GpioAf3); /* PB11ΪLPUART0_RX */

    stc_lpuart_cfg_t  stcCfg;

    DDL_ZERO_STRUCT(stcCfg);

    Sysctrl_SetPeripheralGate(SysctrlPeripheralLpUart0,TRUE); /* ģʱʹ */

    stcCfg.enStopBit = LPUart1bit;                   /* 1ֹͣλ */ 
    stcCfg.enMmdorCk =  LPUartDataOrAddr;            /* ģʽΪ1ʱЧ */  
    stcCfg.stcBaud.enSclkSel = LPUartMskPclk;        /* ʱԴ */
    stcCfg.stcBaud.u32Sclk = Sysctrl_GetPClkFreq();  /* PCLKȡ */
    stcCfg.stcBaud.enSclkDiv = LPUartMsk4Or8Div;     /* ƵĬPCLK4Ƶ */
    stcCfg.stcBaud.u32Baud = baud_rate;              /*    */
    stcCfg.enRunMode = LPUartMskMode1;               /* ģʽ */
    LPUart_Init(M0P_LPUART0, &stcCfg);

    LPUart_ClrStatus(M0P_LPUART0,LPUartRC);          /* ж */
    LPUart_ClrStatus(M0P_LPUART0,LPUartTC);          /* 巢ж */
    LPUart_EnableIrq(M0P_LPUART0,LPUartRxIrq);       /* ʹܽж */
    LPUart_DisableIrq(M0P_LPUART0,LPUartTxIrq);      /* ܷж */
    EnableNvic(LPUART0_IRQn,IrqLevel2,TRUE);         /* ϵͳжʹ */

    p_buf = (uint8_t *)malloc(buf_size);
    if (p_buf == NULL) {
        while(1);
    }

    __g_uart_dev.baud_rate       = baud_rate;
    __g_uart_dev.p_buf           = p_buf;
    __g_uart_dev.buf_size        = buf_size;
    __g_uart_dev.pfn_rx_callback = rx_callback;
    __g_uart_dev.time_out = 150000 / __g_uart_dev.baud_rate + 2;
    __g_uart_init = 1;                                                
}
 
/* 崮ڷͺ */
void mcu_uart_byte_send (uint8_t byte)
{
    LPUart_SendData(M0P_LPUART0, byte);
}

 
/* 崮ڽжϷ */
void LpUart0_IRQHandler(void)
{

    if(LPUart_GetStatus(M0P_LPUART0, LPUartRC))    
    {
        __g_uart_dev.time_out = 150000 / __g_uart_dev.baud_rate + 2;
        
        LPUart_ClrStatus(M0P_LPUART0, LPUartRC);
        __g_uart_dev.p_buf[__g_uart_dev.bytes_cnt] = LPUart_ReceiveData(M0P_LPUART0);
    
        __g_uart_dev.bytes_cnt++;
        if (__g_uart_dev.bytes_cnt > __g_uart_dev.buf_size) {
            __g_uart_dev.bytes_cnt = __g_uart_dev.buf_size - 1;
        }
        
    }

}


/* 崮ڽպ */
int mcu_uart_recv_pkt (uint8_t *p_buf, uint16_t *p_size)
{
    int ret = -1;

    if (__g_uart_dev.bytes_cnt > 0) {
        if (__g_uart_dev.time_out > 0) {
            __delay_ms(5);
            __g_uart_dev.time_out--;
        } else {
            memcpy(p_buf, __g_uart_dev.p_buf, __g_uart_dev.bytes_cnt);
            *p_size = __g_uart_dev.bytes_cnt;
            __g_uart_dev.bytes_cnt = 0;
            ret = 0;        
        }
    }

    return ret;
}






