/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SX126x FSK ģû
 *        ûֲʱֻҪṩSPIGPIOCPUʱɣ
 *        ӿ˵עСýӿڱṩģʾýӿڱṩȷģС
 *
 * \internal
 * \par Modification history
 * - 1.00 22-08-15  sup, first implementation.
 * \endinternal
 */

#include "string.h"
#include "stddef.h"
#include "stdint.h"
#include "radio.h"
#include "stdio.h"
#include "stdlib.h"

#include "ddl.h"

#include "sx126x_fsk.h"

#include "board_gpio.h"
#include "board_spi.h"
#include "radio_int.h"


/**
 * \brief ʵһЩ
 */
static sx126x_spi_funcs_t          __g_spi_dev;
static sx126x_gpio_funcs_t         __g_gpio_dev;
static sx126x_delay_funcs_t        __g_delay;
static sx126x_cpu_lock_funcs_t     __g_cpu_lock;
static radio_sx126x_fsk_dev_t      __g_sx126x;
static radio_sx126x_fsk_devinfo_t  __g_sx126x_devinfo;



/**
 * \brief 豸ͺѡ꣨˴ĺ궨ݲɽ޸ģܵģ޷
 */  
#define  ZSL421     {RADIO_SX126X, OSC_TCXO, 18, 470000000, 510000000}  /* SX126xϵLoRaģ飬ʹԴ֧Ƶ470MHz~510MHz */    



/**
 * \breif FSK 
 */
static sx126x_fsk_param_set_t __g_fsk_param =
{
    ZSL421,                /* 豸ͺѡ񣨱ֻZSL421ģ飩 */
    470000000,             /* Ƶ       [470000000 ~ 510000000]                                 */
    22,                    /* 书    [-9 ~ +22] */
    1,                     /* ʵȼѡ[1 ~ 10]շ˫һ£ */
                           /* 130kb/s   260kb/s   390kb/s   4120kb/s  5 150kb/s 
                              6180kb/s  7210kb/s  8240kb/s  9270kb/s  10300kb/s */
    32,                             /* ǰ볤[1 ~ 65535λbits] */                    
    FSK_PREAMBLE_DETECTOR_BITS_16,  /* ǰ */
                                    /* [FSK_PREAMBLE_DETECTOR_BITS_8 Ϊ8bits]
                                       [FSK_PREAMBLE_DETECTOR_BITS_16Ϊ16bits]
                                       [FSK_PREAMBLE_DETECTOR_BITS_24Ϊ24bits]
                                       [FSK_PREAMBLE_DETECTOR_BITS_32Ϊ32bits] */
                                    /* ע⣺Ͷ˵ǰ볤ȱڻڽն˵ǰ */
    0,                     /* ģʽ[0: , 1ν] */
    100,                   /* νģʽ½ճʱʱ(λms) */
    0,                     /* ͳʱʹ [0:  1: ʹ] */
    FSK_CRC_2_BYTE,        /* ӲCRCУֽշ˫һ£ */
                           /* [FSK_CRC_OFF         CRCܹر]
                            * [FSK_CRC_1_BYTE      1ֽCRC]
                            * [FSK_CRC_2_BYTE      2ֽCRC]
                            * [FSK_CRC_1_BYTE_INV  1ֽCRC㲢ת]
                            * [FSK_CRC_2_BYTE_INV  2ֽCRC㲢ת]
                            * [FSK_CRC_2_BYTES_IBM IBM׼2ֽCRC]
                            * [FSK_CRC_2_BYTES_CCITCCIT׼2ֽCRC]*/
    1,                     /* CRC[0: ر, 1: ]շ˫һ£ */
    1,                     /* ݰ[0ȹ̶1ȿɱ]շ˫һ£ */
    10,                    /* Чس */
};


static void __tx_done_call_func (void)
{
    //printf("__tx_done_call_func!\r\n");
}



static void __tx_timeout_call_func (void)
{
    //printf("__tx_timeout_call_func!\r\n");
}



static void __rx_done_call_func (uint8_t *p_payload, uint16_t size, int16_t rssi, int8_t snr)
{
    printf("pkt rssi: %d    snr: %d  \r\n", rssi, snr);
}



static void __rx_timeout_call_func (void)
{
    //printf("__rx_timeout_call_func!\r\n");
}


static void __rx_hw_crc_err_call_func (void)
{
    //printf("__rx_hw_crc_err_call_func!\r\n"); 
}


static void __rx_sw_crc_err_call_func (void)
{
    //printf("__rx_sw_crc_err_call_func!\r\n"); 
}





/**
 * \brief ¼ص
 */
static sx126x_fsk_events_t  __g_sx126x_events = {
    __tx_done_call_func,
    __tx_timeout_call_func,
    __rx_done_call_func,
    __rx_timeout_call_func,
    __rx_hw_crc_err_call_func,
    __rx_sw_crc_err_call_func,
};


radio_handle_t radio_fsk_zm4xx_inst_init (void)
{
    /********** SPI д **********/

    /* SPI һֽںýӿڱṩ */
    __g_spi_dev.pfn_spi_write_byte = mcu_spi_send_byte;

    /* SPI һֽںýӿڱṩ */
    __g_spi_dev.pfn_spi_read_byte  = mcu_spi_recv_byte;

    
    /********** GPIO  **********/

    /* SX_NRESET λŵƽáýӿڱṩ */    
    __g_gpio_dev.pfn_reset_pin_set = mcu_gpio_rst_pin_set; 

    /* SPI_NSS Ƭѡŵƽáýӿڱṩ */ 
    __g_gpio_dev.pfn_sel_pin_set = mcu_gpio_sel_pin_set;
    
    /*   
     * Ƶطŵƽáýӿڿɸģѡṩ
     * ΪZSL420ZSL421ZM68Sϵģ飬ṩ˽ӿ
     * ΪZSL64ϵģ飬ֱô˽ӿΪNULL߽ӿʵΪ
     */
    __g_gpio_dev.pfn_txen_pin_set = mcu_gpio_txen_pin_set;  

    /* VCTL_1(RXEN) Ƶؽտŵƽáýӿڱṩ */
    __g_gpio_dev.pfn_rxen_pin_set = mcu_gpio_rxen_pin_set;
    
    /*  BUSY æŵƽȡýӿڱṩ */
    __g_gpio_dev.pfn_busy_pin_read = mcu_gpio_busy_pin_read;



    /********** ʱ **********/

    /* ʱýӿڱṩ */
    __g_delay.pfn_delay_ms = delay1ms;

 
 
    /********** CPU **********/
    /* ĺڱSPI䣬ܻᵼģ */
 
    /* CPUжϡýӿڱṩ */
    __g_cpu_lock.pfn_cpu_lock   = cpu_lock;
    /* CPUжϡýӿڱṩ */
    __g_cpu_lock.pfn_cpu_unlock = cpu_unlock;
 

    /* 豸Ϣ */
    __g_sx126x_devinfo.p_events     = &__g_sx126x_events;
    __g_sx126x_devinfo.p_fsk_param = &__g_fsk_param;

    return radio_sx126x_fsk_init(&__g_sx126x,
                                 &__g_spi_dev,
                                 &__g_gpio_dev,
                                 &__g_delay,
                                 &__g_cpu_lock,
                                 &__g_sx126x_devinfo);
}


