/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SPI
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
#include "stdio.h"

#include "gpio.h"
#include "spi.h"
#include "sysctrl.h"
#include "reset.h"
#include "ddl.h"

#include "board_gpio.h"
#include "board_spi.h"




#define  CLK_L()     (Gpio_WriteOutputIO(GpioPortA, GpioPin5, 0))       /* SCKߵƽ */
#define  CLK_H()     (Gpio_WriteOutputIO(GpioPortA, GpioPin5, 1))       /* SCK͵ƽ */

#define  MOSI_H()    (Gpio_WriteOutputIO(GpioPortA, GpioPin7, 1))       /* MOSIߵƽ */
#define  MOSI_L()    (Gpio_WriteOutputIO(GpioPortA, GpioPin7, 0))       /* MOSI͵ƽ */

#define  MISO_READ() (Gpio_GetInputIO(GpioPortA, GpioPin6))             /* MISOƽȡ */





#define        TIME        10



#if    HARDWARE_SPI  

/* SPIһֽںýӿڱṩ */
void mcu_spi_send_byte (uint8_t send_data)
{
    M0P_SPI_TypeDef *SPIx = M0P_SPI0;

    while(SPIx->STAT_f.TXE == 0){;}
    M0P_SPI0->DATA = send_data; 

    while(SPIx->STAT_f.RXNE == 0){;}
    SPIx->DATA;

    while(TRUE == SPIx->STAT_f.BUSY){;}

}

/* SPIһֽںýӿڱṩ */
uint8_t mcu_spi_recv_byte (void)
{
    M0P_SPI_TypeDef *SPIx = M0P_SPI0;

    uint8_t buffer;

    while(SPIx->STAT_f.TXE == 0){;}
    M0P_SPI0->DATA = 0x00;  

    while(TRUE == SPIx->STAT_f.BUSY){;}

    while(SPIx->STAT_f.RXNE == 0){;}
    buffer = SPIx->DATA;

    return buffer;
}

/* SPIʼ */
int mcu_spi_init (void)
{
    stc_gpio_cfg_t GpioInitStruct;
    DDL_ZERO_STRUCT(GpioInitStruct);

    Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);

    GpioInitStruct.enDrv = GpioDrvH;     /* GPIO */
    GpioInitStruct.enDir = GpioDirOut;   /* GPIO      */

    GpioInitStruct.enOD = GpioOdDisable; /*  */
    GpioInitStruct.enPu = GpioPuEnable;  /*  */
    GpioInitStruct.enPd = GpioPdDisable; /*  */

    /* SPI0_CS  PIOA_15 */
    Gpio_Init(GpioPortA, GpioPin15, &GpioInitStruct);    
    Gpio_SetAfMode(GpioPortA, GpioPin15, GpioAf1);                                                            

    /* SPI0_SCK  PIOA_5 */
    Gpio_Init(GpioPortA, GpioPin5, &GpioInitStruct);
    Gpio_SetAfMode(GpioPortA, GpioPin5, GpioAf1);                                                       

    /* SPI0_MOSI  PIOA_7 */
    Gpio_Init(GpioPortA, GpioPin7, &GpioInitStruct);
    Gpio_SetAfMode(GpioPortA, GpioPin7, GpioAf1);                                                             

    /* SPI0_MISO  PIOA_6 */
    GpioInitStruct.enDir = GpioDirIn;    /* GPIO  */       
    GpioInitStruct.enPu = GpioPuEnable;  /*  */
    GpioInitStruct.enPd = GpioPdDisable; /*  */
    Gpio_Init(GpioPortA, GpioPin6, &GpioInitStruct);            
    Gpio_SetAfMode(GpioPortA, GpioPin6, GpioAf1);        

    stc_spi_cfg_t  SpiInitStruct;  

    Sysctrl_SetPeripheralGate(SysctrlPeripheralSpi0,TRUE);

    Reset_RstPeripheral0(ResetMskSpi0);         /* λģ */

    SpiInitStruct.enSpiMode = SpiMskMaster;     /* λģʽ */
    SpiInitStruct.enPclkDiv = SpiClkMskDiv32;   /*         */
    SpiInitStruct.enCPHA    = SpiMskCphafirst;  /* һزCPHA=0 */
    SpiInitStruct.enCPOL    = SpiMskcpollow;    /* Ϊͣ    CPOL=0 */
    Spi_Init(M0P_SPI0, &SpiInitStruct);

    return 0;
}



#else



static void __delay_us (unsigned int i)
{
    unsigned int j, k;

    for (j = 0; j < 1; j++){
        for (k = 0; k < i; k++);
    }
}


/* SPIһֽںýӿڱṩ */
void mcu_spi_send_byte (uint8_t send_data)
{
    uint8_t i;

    for (i = 0; i < 8; i++) {
        __delay_us(TIME);
        CLK_L();                                                         /* clk=0 */
        if((send_data << i) & 0x80){
            __delay_us(TIME);
            MOSI_H();                                                    /* mosi=1 */
        } else {
            __delay_us(TIME);
            MOSI_L();                                                    /* mosi=0 */
        }

        __delay_us(TIME);
        CLK_H();                                                         /* clk=1 */
    }

    __delay_us(TIME);
    CLK_L();                                                             /* clk=0 */
    MOSI_L();                                                            /* mosi=0 */
}

/* SPIһֽںýӿڱṩ */
uint8_t mcu_spi_recv_byte (void)
{
    uint8_t i, temp;

    temp = 0;
    for (i = 0; i < 8; i++) {
        CLK_L();                                                         /* clk=0 */
        __delay_us(TIME);
        temp = (temp << 1);
        CLK_H();                                                         /* clk=1 */
        __delay_us(TIME);
        if (MISO_READ()) {
            temp++;
        } else {                                                          
            temp = temp + 0;
        }

        __delay_us(TIME);
    }

    CLK_L();                                                              /* clk=0 */
    __delay_us(TIME);

    return temp;
}

/* SPIʼ */
int mcu_spi_init (void)
{

    stc_gpio_cfg_t GpioInitStruct;
    DDL_ZERO_STRUCT(GpioInitStruct);


    /* MOSI  PA07  Ĭ͵ƽ */
    GpioInitStruct.enDir = GpioDirOut;      
    GpioInitStruct.enPu  = GpioPuDisable; 
    GpioInitStruct.enPd  = GpioPdEnable;   
    Gpio_WriteOutputIO(GpioPortA, GpioPin7, 0); 
    Gpio_Init(GpioPortA, GpioPin7, &GpioInitStruct);


    /* MISO  PA06    */
    GpioInitStruct.enDir = GpioDirIn;
    GpioInitStruct.enPu  = GpioPuDisable;
    GpioInitStruct.enPd  = GpioPdEnable;   
    Gpio_Init(GpioPortA, GpioPin6, &GpioInitStruct);


    /* PA05  SCK  Ĭ͵ƽ */
    GpioInitStruct.enDir = GpioDirOut;     
    GpioInitStruct.enPu  = GpioPuDisable; 
    GpioInitStruct.enPd  = GpioPdEnable;   
    Gpio_WriteOutputIO(GpioPortA, GpioPin5, 0); 
    Gpio_Init(GpioPortA, GpioPin5, &GpioInitStruct);


    /* PA15  CS  Ĭߵƽ */
    GpioInitStruct.enDir = GpioDirOut;
    GpioInitStruct.enPu  = GpioPuEnable;    
    GpioInitStruct.enPd  = GpioPdDisable;   
    Gpio_WriteOutputIO(GpioPortA, GpioPin15, 1);  
    Gpio_Init(GpioPortA, GpioPin15, &GpioInitStruct);

    return 0;
    
}



#endif












