/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SX126xģû
 *        ûֲʱֻҪṩSPIGPIOCPUʱɣ
 *        ӿ˵עСýӿڱṩģʾýӿڱṩȷģС
 *
 * \internal
 * \par Modification history
 * - 1.00 19-09-09  sup, first implementation.
 * \endinternal
 */

#include "string.h"
#include "stddef.h"
#include "stdint.h"
#include "radio.h"
#include "stdio.h"
#include "stdlib.h"

#include "ddl.h"

#include "sx126x_lora.h"

#include "board_gpio.h"
#include "board_spi.h"
#include "radio_int.h"



/**
 * \brief ʵһЩ̬ȫֱ 
 */ 
static sx126x_spi_funcs_t           __g_spi_dev;
static sx126x_gpio_funcs_t          __g_gpio_dev;
static sx126x_delay_funcs_t         __g_delay;
static sx126x_cpu_lock_funcs_t      __g_cpu_lock;
static radio_sx126x_lora_dev_t      __g_sx126x;
static radio_sx126x_lora_devinfo_t  __g_sx126x_devinfo;  





/**
 * \brief 豸ͺѡ񣨴˴ĺ궨ݲɽ޸ģܵģ޷
 */
#define  ZSL420     {RADIO_SX126X, OSC_XTAL, 15, 470000000, 510000000}  /* SX126xϵLoRaģ飬ʹԴ֧Ƶ470MHz~510MHz */    



/**
 * \breif
 * LoRa   
 */
static sx126x_lora_param_set_t __g_lora_param =
{
    ZSL420,           /* 豸ͺѡ񣨱ֻZSL420ģ飩 */
    470000000,        /* Ƶ       [470000000 ~ 510000000]                                 */
    22,               /* 书   [-9 ~ +22]                                             */
    8,                /*        [6: 62.5 kHz, 7: 125 kHz, 8: 250 kHz,  9: 500 kHz]      */
    7,                /* Ƶ   [532, 6: 64, 7: 128, 8: 256, 9: 512, 10: 1024, 11: 2048, 12: 4096 chips] */
    1,                /*         [1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8]                                 */
    1,                /* ӲCRC   [0: ر, 1: ]                                               */
    0,                /* ͷģʽ      [0: ʽͷģʽ, 1: ʽͷģʽ]                               */
    0,                /* ģʽ      [0: Continuous, 1: Single]                                       */
    100,              /* νģʽ½ճʱʱ(ms)                                                 */
    0,                /* ͳʱʹ[0:  1: ʹ]                                                  */
    10,               /* س                                                                       */
    16,               /* ǰ볤                                                                     */
    1,                /* IQת       [0: , 1: ת]                                                */
    0,                /* ʹ     [0ܣʹ˽1ʹܣʹù]                            */
    1,                /* CRC   [0: ر, 1: ]                                               */ 
    1,                /* Żʽѡ[0: ֶ, 1: Զ] */
    1,                /* ֶƵŻĿ(ǰʽѡΪ"ֶ")[0: ر, 1: ] */
};






static void __tx_done_call_func (void)
{
    //printf("__tx_done_call_func!\r\n");
}



static void __tx_timeout_call_func (void)
{
    //printf("__tx_timeout_call_func!\r\n");
}



static void __rx_done_call_func (uint8_t *p_payload, uint16_t size, int16_t rssi, int8_t snr)
{
    printf("pkt rssi: %d    snr: %d  \r\n", rssi, snr);
}



static void __rx_timeout_call_func (void)
{
    //printf("__rx_timeout_call_func!\r\n");
}


static void __rx_hw_crc_err_call_func (void)
{
    //printf("__rx_hw_crc_err_call_func!\r\n"); 
}


static void __rx_sw_crc_err_call_func (void)
{
    //printf("__rx_sw_crc_err_call_func!\r\n"); 
}


static void __cad_done_call_func (bool ch_active)
{
    //printf("__cad_done_call_func!\r\n");
}    
    


/**
 * \brief ¼ص
 */
static sx126x_lora_events_t  __g_sx126x_events = {
    __tx_done_call_func,
    __tx_timeout_call_func,
    __rx_done_call_func,
    __rx_timeout_call_func,
    __rx_hw_crc_err_call_func,
    __rx_sw_crc_err_call_func,
    __cad_done_call_func
};



radio_handle_t radio_lora_zm4xx_inst_init (void)
{
    /********** SPI д **********/

    /* SPI һֽںýӿڱṩ */
    __g_spi_dev.pfn_spi_write_byte = mcu_spi_send_byte;

    /* SPI һֽںýӿڱṩ */
    __g_spi_dev.pfn_spi_read_byte  = mcu_spi_recv_byte;
    
    
    /********** GPIO  **********/

    /* SX_NRESET λŵƽáýӿڱṩ */    
    __g_gpio_dev.pfn_reset_pin_set = mcu_gpio_rst_pin_set; 

    /* SPI_NSS Ƭѡŵƽáýӿڱṩ */ 
    __g_gpio_dev.pfn_sel_pin_set = mcu_gpio_sel_pin_set;
    
    /*   
     * Ƶطŵƽáýӿڿɸģѡṩ
     * ΪZSL420ZSL421ZM68Sϵģ飬ṩ˽ӿ
     * ΪZSL64ϵģ飬ֱô˽ӿΪNULL߽ӿʵΪ
     */
    __g_gpio_dev.pfn_txen_pin_set = mcu_gpio_txen_pin_set;  

    /* VCTL_1(RXEN) Ƶؽտŵƽáýӿڱṩ */
    __g_gpio_dev.pfn_rxen_pin_set = mcu_gpio_rxen_pin_set;
    
    /*  BUSY æŵƽȡýӿڱṩ */
    __g_gpio_dev.pfn_busy_pin_read = mcu_gpio_busy_pin_read;



    /********** ʱ **********/

    /* ʱýӿڱṩ */
    __g_delay.pfn_delay_ms = delay1ms;

 
 
    /********** CPU **********/
    /* ĺڱSPI䣬ܻᵼģ */
 
    /* CPUжϡýӿڱṩ */
    __g_cpu_lock.pfn_cpu_lock   = cpu_lock;
    /* CPUжϡýӿڱṩ */
    __g_cpu_lock.pfn_cpu_unlock = cpu_unlock;
 

    /* 豸Ϣ */
    __g_sx126x_devinfo.p_events     = &__g_sx126x_events;
    __g_sx126x_devinfo.p_lora_param = &__g_lora_param;

    return radio_sx126x_lora_init(&__g_sx126x,
                                  &__g_spi_dev,
                                  &__g_gpio_dev,
                                  &__g_delay,
                                  &__g_cpu_lock,
                                  &__g_sx126x_devinfo);
}







