/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

#include "radio.h"
#include "string.h"




int radio_reset (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_reset(handle->p_drv);
}



int radio_sync_id_set (radio_handle_t handle, uint8_t *p_id, uint8_t len)
{
    if (handle == NULL || p_id == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_id_set(handle->p_drv, p_id, len);
}



int radio_sync_id_get (radio_handle_t handle, uint8_t *p_id, uint8_t *p_len)
{
    if (handle == NULL || p_id == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_id_get(handle->p_drv, p_id, p_len);
}



int radio_preamble_length_set (radio_handle_t handle, uint16_t size)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_preamble_length_set(handle->p_drv, size);
}



int radio_preamble_length_get (radio_handle_t handle, uint16_t *p_len)
{   
    return handle->p_funcs->pfn_radio_preamble_length_get(handle->p_drv, p_len);
}



int radio_tx_power_set (radio_handle_t handle, uint8_t level, float *p_power)
{
    if (handle == NULL ) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_tx_power_set(handle->p_drv, level, p_power);
}



int radio_freq_set (radio_handle_t handle, uint32_t freq)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_freq_set(handle->p_drv, freq);
}



int radio_cur_rssi_get (radio_handle_t handle, int16_t *p_rssi)
{
    if ((handle == NULL) || (p_rssi == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_cur_rssi_get(handle->p_drv, p_rssi);
}



int radio_mode_set (radio_handle_t handle, uint8_t mode)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_mode_set(handle->p_drv, mode);
}



int radio_state_get (radio_handle_t handle, radio_state_t *p_state)
{
    if ((handle == NULL) || (p_state == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_state_get(handle->p_drv, p_state);
}



int radio_data_send (radio_handle_t handle, uint8_t *buf, uint8_t size)
{
    if ((handle == NULL) || (buf == NULL) || (size == 0)) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_data_send(handle->p_drv, buf, size);
}



int radio_data_recv (radio_handle_t handle, uint8_t *buf, uint8_t *p_size)
{
    if ((handle == NULL) || (buf == NULL) || (p_size == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }

    return handle->p_funcs->pfn_radio_data_recv(handle->p_drv, buf, p_size);
}



int radio_time_on_air_get (radio_handle_t handle, 
                           uint16_t       preamble_len,
                           uint8_t        packet_len,
                           uint32_t      *p_time)
{
    if ((handle == NULL) || (p_time == NULL)) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    return handle->p_funcs->pfn_radio_time_on_air_get(
                                handle->p_drv, 
                                preamble_len,
                                packet_len,
                                p_time);
}



int radio_ioctl (radio_handle_t handle, int cmd, void *p_arg)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    } 
    
    return handle->p_funcs->pfn_radio_ioctl(handle->p_drv, cmd, p_arg);
}




int radio_dio0_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio0_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio0_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio1_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio1_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio1_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio2_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio2_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio2_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio3_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio3_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio3_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}




int radio_dio5_irq_func (radio_handle_t handle)
{
    if (handle == NULL) {
        return RADIO_RET_PARAM_INVALID;
    }
    
    if (handle->p_funcs->pfn_radio_dio5_irq_func != NULL) {
        handle->p_funcs->pfn_radio_dio5_irq_func(handle->p_drv);
		return RADIO_RET_OK;
    } else {
		return RADIO_RET_MODULE_NOT_SUPPORT;
	}
}
