/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/


#ifndef __RADIO_H_
#define __RADIO_H_

#include "stdint.h"
#include "radio_differ.h"


/**
 * \brief ģʽ
 */
#define SLEEP_MODE                       0x00 /* ģʽ */
#define STANDBY_MODE                     0x01 /* ģʽ */
#define RX_MODE                          0x02 /* ģʽ */
#define CW_MODE                          0x03 /* زģʽ */

/**
 * \brief ǰ
 */
#define RADIO_TIME_ON_AIR_USE_CUR_PREAMBLE_LEN  0xFFFF

/**
 * \brief ӿڷֵ
 */
#define RADIO_RET_OK                      0
#define RADIO_RET_ERR                    -1 
#define RADIO_RET_CMD_NOT_SUPPORT        -2
#define RADIO_RET_PARAM_INVALID          -3
#define RADIO_RET_CFG_OUT_OF_RANGE       -4
#define RADIO_RET_FREQ_NOT_SUPPORT       -5
#define RADIO_RET_MODULE_NOT_SUPPORT     -6
#define RADIO_RET_NOT_RECV_PKT           -7
#define RADIO_RET_NOT_IN_RX_MODE         -8
#define RADIO_RET_TX_TIMEOUT             -9
#define RADIO_RET_RX_TIMEOUT             -10
#define RADIO_RET_RX_CRC_ERR             -11


/**
 * \brief ģָ
 */
struct radio_drv_funcs {
    int  (*pfn_radio_reset)               (void *p_drv);
    int  (*pfn_radio_id_set)              (void *p_drv, uint8_t *p_id, uint8_t id_len);
    int  (*pfn_radio_id_get)              (void *p_drv, uint8_t *p_id, uint8_t *id_len);
    int  (*pfn_radio_preamble_length_set) (void *p_drv, uint16_t size);
    int  (*pfn_radio_preamble_length_get) (void *p_drv, uint16_t *p_len);
    int  (*pfn_radio_tx_power_set)        (void *p_drv, uint8_t level, float *p_power);
    int  (*pfn_radio_freq_set)            (void *p_drv, uint32_t freq);
    int  (*pfn_radio_mode_set)            (void *p_drv, uint8_t mode);
    int  (*pfn_radio_state_get)           (void *p_drv, radio_state_t *p_state);
    int  (*pfn_radio_cur_rssi_get)        (void *p_drv, int16_t *p_rssi);
    int  (*pfn_radio_time_on_air_get)     (void     *p_drv, 
                                           uint16_t  preamble_len,
                                           uint8_t   packet_len,
                                           uint32_t *p_time);
    int  (*pfn_radio_data_send)           (void *p_drv, uint8_t *p_buf, uint8_t size);
    int  (*pfn_radio_data_recv)           (void *p_drv, uint8_t *p_buf, uint8_t *p_size);
    int  (*pfn_radio_ioctl)               (void *p_drv, int cmd, void *p_arg);
    void (*pfn_radio_dio0_irq_func)       (void *p_drv);
    void (*pfn_radio_dio1_irq_func)       (void *p_drv);
    void (*pfn_radio_dio2_irq_func)       (void *p_drv);
    void (*pfn_radio_dio3_irq_func)       (void *p_drv);
    void (*pfn_radio_dio5_irq_func)       (void *p_drv);
};



/**
 * \brief Ƶ豸
 */
typedef struct radio_dev {
    const struct radio_drv_funcs *p_funcs;   /* 豸  */
    void                         *p_drv;
} radio_dev_t;



typedef radio_dev_t *radio_handle_t;



/**
 * \brief ģ鸴λ
 *
 * \note λҪģ³ʼ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK            : λɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 */
int radio_reset (radio_handle_t handle);



/**
 * \brief ģͬID
 *
 * \note ǰȰģлSTANDBYģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_id     : IDݵָ
 * \param[in]  id_len   : ID
 *
 * \retval  RADIO_RET_OK                 : óɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE   : IDȳΧ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ù
 */
int radio_sync_id_set (radio_handle_t handle, uint8_t *p_id, uint8_t id_len);



/**
 * \brief ȡģͬID
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_id     : IDݵָ
 * \param[in]  p_len    : ID
 *
 * \retval  RADIO_RET_OK                 : ȡɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ù
 */
int radio_sync_id_get (radio_handle_t handle, uint8_t *p_id, uint8_t *p_len);



/**
 * \brief ģǰ볤
 *
 * \note ǰȰģлSTANDBYģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  size     : ǰ볤
 *
 * \retval  RADIO_RET_OK               : óɹ
 * \retval  RADIO_RET_PARAM_INVALID    : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE : ǰ볤ȳΧ
 */
int radio_preamble_length_set (radio_handle_t handle, uint16_t size);



/**
 * \brief ȡģǰ볤
 *
 * \param[in]  handle   : ģ
 * \param[out] p_len    : ǰ볤
 *
 * \retval  RADIO_RET_OK            : ȡɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 */
int radio_preamble_length_get (radio_handle_t handle, uint16_t *p_len);



/**
 * \brief ģ鷢书ʵλ0~7
 *
 * \note ǰȰģлSTANDBYģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  level    : ʵλ
 * \param[out] p_power  : ʵʵù
 *
 * \retval  RADIO_RET_OK            : óɹ
 * \retval  RADIO_RET_ERR           : ʧ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 */
int radio_tx_power_set (radio_handle_t handle, uint8_t level, float *p_power);



/**
 * \brief ģƵ
 *
 * \note ǰȰģлSTANDBYģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  freq     : Ƶ
 *
 * \retval  RADIO_RET_OK               : óɹ
 * \retval  RADIO_RET_PARAM_INVALID    : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE : ƵʳΧ
 * \retval  RADIO_RET_FREQ_NOT_SUPPORT : ָ֧Ƶ
 */
int radio_freq_set (radio_handle_t handle, uint32_t freq);



/**
 * \brief ģ鵱ǰRSSIֵ
 *
 * \param[in]  handle : ģ
 * \param[out] p_rssi : RSSIֵ
 *
 * \retval RADIO_RET_OK            : RSSIֵȡɹ
 * \retval RADIO_RET_PARAM_INVALID : Ч
 */
int radio_cur_rssi_get (radio_handle_t handle, int16_t *p_rssi);



/**
 * \brief ģģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  mode     : ģʽ
 *
 * \retval  RADIO_RET_OK                 : óɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ģʽ
 */
int radio_mode_set (radio_handle_t handle, uint8_t mode);



/**
 * \brief ȡģ鵱ǰ״̬
 *
 * \param[in]  handle  : ģ
 * \param[out] p_state : ģ״̬
 *
 * \retval  RADIO_RET_OK                 : ȡɹ
 * \retval  RADIO_RET_PARAM_INVALID      : Ч
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģ鲻ָ֧ģʽ
 */
int radio_state_get (radio_handle_t handle, radio_state_t *p_state);



/**
 * \brief ģ鷢
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_buf    : ݻ
 * \param[in]  size     : ݳ
 *
 * \retval  RADIO_RET_OK               : ͳɹ
 * \retval  RADIO_RET_PARAM_INVALID    : Ч
 * \retval  RADIO_RET_CFG_OUT_OF_RANGE : ȳΧ
 */
int radio_data_send (radio_handle_t handle, uint8_t *p_buf, uint8_t size);



/**
 * \brief ģ
 *
 * \param[in]  handle   : ģ
 * \param[in]  p_buf    : ݻ
 * \param[in]  size     : յݵĳ
 *
 * \retval  RADIO_RET_OK            : ճɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 * \retval  RADIO_RET_NOT_RECV_PKT  : ûнյݰ
 */
int radio_data_recv (radio_handle_t handle, uint8_t *p_buf, uint8_t *p_size);



/**
 * \brief ģ
 *
 * \param[in]  handle       : ģ
 * \param[in]  preamble_len : ǰ볤ȣʹõǰǰ볤ʹú
                                RADIO_TIME_ON_AIR_USE_CUR_PREAMBLE_LEN
 * \param[in]  pkt_len      : 
 * \param[out] p_time       : ʱ
 *
 * \retval  RADIO_RET_OK            : ճɹ
 * \retval  RADIO_RET_PARAM_INVALID : Ч
 * \retval  RADIO_RET_NOT_RECV_PKT  : ûнյݰ
 */
int radio_time_on_air_get (radio_handle_t handle, 
                           uint16_t       preamble_len,
                           uint8_t        pkt_len,
                           uint32_t      *p_time);



/**
 * \brief ģƺ
 *
 * \note ǰȰģлSTANDBYģʽ
 *
 * \param[in]  handle   : ģ
 * \param[in]  cmd      : 
 * \param[in]  p_arg    : ӦĲ
 *
 * \retval  RADIO_RET_OK              : ִгɹ
 * \retval  RADIO_RET_ERR             : ִʧ
 * \retval  RADIO_RET_PARAM_INVALID   : Ч
 * \retval  RADIO_RET_CMD_NOT_SUPPORT : ָ֧
 */
int radio_ioctl (radio_handle_t handle, int cmd, void *p_arg);



/**
 * \brief ģDIO0жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio0_irq_func (radio_handle_t handle);
 
 
 
/**
 * \brief ģDIO1жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio1_irq_func (radio_handle_t handle);
 
 
 
/**
 * \brief ģDIO2жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio2_irq_func (radio_handle_t handle);
 

 
/**
 * \brief ģDIO3жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio3_irq_func (radio_handle_t handle);
 
 
 
/**
 * \brief ģDIO5жϷ
 *
 * \param[in]  handle   : ģ
 *
 * \retval  RADIO_RET_OK                 : жϷִ
 * \retval  RADIO_RET_MODULE_NOT_SUPPORT : ģжϷ
 */
int radio_dio5_irq_func (radio_handle_t handle);
 
#endif /* _RADIO_H_ */
