/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SX126x FSK driver
 *
 * \internal
 * \par Modification history
 * - 1.00 22-08-15  sup, first implementation.
 * \endinternal
 */

#ifndef SX126X_FSK_H_
#define SX126X_FSK_H_


#include "stdbool.h"
#include "radio.h"
#include "sx126x_common.h"


/**
 * \brief FSKģʽؼĴ
 */
#define    REG_ADDR_FSK_WHITEN_INIT_MSB    0x06B8  /* ׻ʼֵMSBĴ */
#define    REG_ADDR_FSK_WHITEN_INIT_LSB    0x06B9  /* ׻ʼֵLSBĴ */
#define    REG_ADDR_FSK_CRC_INIT_MSB       0x06BC  /* CRCʼֵMSBĴ */
#define    REG_ADDR_FSK_CRC_INIT_LSB       0x06BD  /* CRCʼֵLSBĴ */
#define    REG_ADDR_FSK_CRC_POLY_MSB       0x06BE  /* CRCʽֵMSBĴ */
#define    REG_ADDR_FSK_CRC_POLY_LSB       0x06BF  /* CRCʽֵLSBĴ */
#define    REG_ADDR_FSK_SYNCWORD0          0x06C0  /* ͬ0Ĵ */
#define    REG_ADDR_FSK_SYNCWORD1          0x06C1  /* ͬ1Ĵ */
#define    REG_ADDR_FSK_SYNCWORD2          0x06C2  /* ͬ2Ĵ */
#define    REG_ADDR_FSK_SYNCWORD3          0x06C3  /* ͬ3Ĵ */
#define    REG_ADDR_FSK_SYNCWORD4          0x06C4  /* ͬ4Ĵ */
#define    REG_ADDR_FSK_SYNCWORD5          0x06C5  /* ͬ5Ĵ */
#define    REG_ADDR_FSK_SYNCWORD6          0x06C6  /* ͬ6Ĵ */
#define    REG_ADDR_FSK_SYNCWORD7          0x06C7  /* ͬ7Ĵ */
#define    REG_ADDR_FSK_NODE_ADDR          0x06CD  /* ڵַĴ */
#define    REG_ADDR_FSK_BROADCAST_ADDR     0x06CE  /* 㲥ַĴ */



 /**
  * \brief FSKǰ
  */
#define    FSK_PREAMBLE_DETECTOR_OFF        0x00  /* رǰ */
#define    FSK_PREAMBLE_DETECTOR_BITS_8     0x04  /* 8 bits */
#define    FSK_PREAMBLE_DETECTOR_BITS_16    0x05  /* 16 bits */
#define    FSK_PREAMBLE_DETECTOR_BITS_24    0x06  /* 24 bits */
#define    FSK_PREAMBLE_DETECTOR_BITS_32    0x07  /* 32 bits */

/**
 * \brief FSKַ˹ѡ
 */
#define    FSK_ADDR_FILTER_DISABLE           0x00  /* ַ˹ܹر */
#define    FSK_ADDR_FILTER_NODE              0x01  /* --˽ڵַ  */
#define    FSK_ADDR_FILTER_NODE_BROADCAST    0x02  /* --˽ڵ㡢㲥ַ */

/**
 * \brief CRCѡ
 */
#define    FSK_CRC_OFF             0x01  /* CRCܹر */
#define    FSK_CRC_1_BYTE          0x00  /* 1ֽڵCRC */
#define    FSK_CRC_2_BYTE          0x02  /* 2ֽڵCRC */
#define    FSK_CRC_1_BYTE_INV      0x04  /* 1ֽڵCRC㲢ת */
#define    FSK_CRC_2_BYTE_INV      0x06  /* 2ֽڵCRC㲢ת */
#define    FSK_CRC_2_BYTES_IBM     0x07  /* IBM׼2ֽCRC */
#define    FSK_CRC_2_BYTES_CCIT    0x08  /* CCIT׼2ֽCRC */


/**
 * \brief IBM׼CRCʼֵʽ
 */
#define    FSK_CRC_INIT_VAL_IBM      0xFFFF  /* IBM׼CRCLFSRʼֵ */
#define    FSK_CRC_POLYNOMIAL_IBM    0x8005  /* IBM׼CRCĶʽ */

/**
 * \brief CCIT׼CRCʼֵʽ
 */
#define    FSK_CRC_INIT_VAL_CCIT      0x1D0F  /* CCIT׼CRCLFSRʼֵ */
#define    FSK_CRC_POLYNOMIAL_CCIT    0x1021  /* CCIT׼CRCĶʽ */


/**
 * \brief FSK˲ѡö 
 */
#define    FSK_FILTER_NO          0x00  /* ˲ */
#define    FSK_FILTER_GS_BT_03    0x08  /* ˹˲ BT=0.3 */
#define    FSK_FILTER_GS_BT_05    0x09  /* ˹˲ BT=0.5 */
#define    FSK_FILTER_GS_BT_07    0x0A  /* ˹˲ BT=0.7 */
#define    FSK_FILTER_GS_BT_1     0x0B  /* ˹˲ BT=1.0 */

/**
 * \brief FSKֵ 
 */
#define    FSK_RX_BW_4800      0x1F  /* մ4.8kHz */
#define    FSK_RX_BW_5800      0x17  /* մ5.8kHz */
#define    FSK_RX_BW_7300      0x0F  /* մ7.3Hz */
#define    FSK_RX_BW_9700      0x1E  /* մ9.7kHz */
#define    FSK_RX_BW_11700     0x16  /* մ11.7kHz */
#define    FSK_RX_BW_14600     0x0E  /* մ14.6kHz */
#define    FSK_RX_BW_19500     0x1D  /* մ19.5kHz */
#define    FSK_RX_BW_23400     0x15  /* մ23.4kHz */
#define    FSK_RX_BW_29300     0x0D  /* մ29.3kHz */
#define    FSK_RX_BW_39000     0x1C  /* մ39.0kHz */
#define    FSK_RX_BW_46900     0x14  /* մ46.9kHz */
#define    FSK_RX_BW_58600     0x0C  /* մ58.6kHz */
#define    FSK_RX_BW_78200     0x1B  /* մ78.2kHz */
#define    FSK_RX_BW_93800     0x13  /* մ93.8kHz */
#define    FSK_RX_BW_117300    0x0B  /* մ117.3kHz */
#define    FSK_RX_BW_156200    0x1A  /* մ156.2kHz */
#define    FSK_RX_BW_187200    0x12  /* մ187.2kHz */
#define    FSK_RX_BW_234300    0x0A  /* մ234.3kHz */
#define    FSK_RX_BW_312000    0x19  /* մ312.0kHz */
#define    FSK_RX_BW_373600    0x11  /* մ373.6kHz */
#define    FSK_RX_BW_467000    0x09  /* մ467.0kHz */


/**
 * \brief FSKϢṹ 
 */
typedef struct sx126x_fsk_bw {
    uint32_t bw_value;
    uint8_t  bw_cmd;
} sx126x_fsk_bw_t;


/**
 * \brief FSKʵȼϢṹ
 */
typedef struct sx126x_fsk_rate_level{
    uint32_t bit_rate;           
    uint32_t freq_dev;           
    uint32_t bandwidth;          
} sx126x_fsk_rate_level_t;


/**
 * \brief FSK
 */
typedef struct sx126x_fsk_param_set {
    rf_dev_t dev_type;           /* 豸ͺѡ */
    uint32_t frequency;          /* Ƶ        [470000000 ~ 510000000]շ˫һ£ */
    int8_t   power;              /* 书    [-9 ~ +22] */
    uint8_t  rate_level;         /* ʵȼѡ[1 ~ 10]շ˫һ£ */
                                 /* 130kb/s   260kb/s   390kb/s   4120kb/s  5 150kb/s */
                                 /* 6180kb/s  7210kb/s  8240kb/s  9270kb/s  10300kb/s */
    uint16_t preamble_len;       /* ǰ볤[1 ~ 65535λbits] */
    uint8_t  preamble_det_len;   /* ǰ */
                                 /* [FSK_PREAMBLE_DETECTOR_BITS_8 Ϊ8bits]
                                    [FSK_PREAMBLE_DETECTOR_BITS_16Ϊ16bits]
                                    [FSK_PREAMBLE_DETECTOR_BITS_24Ϊ24bits]
                                    [FSK_PREAMBLE_DETECTOR_BITS_32Ϊ32bits] */
                                 /* ע⣺Ͷ˵ǰ볤ȱڻڽն˵ǰ */
    uint8_t  rx_mode;            /* ģʽ[0: , 1ν] */
    uint32_t rx_pkt_timeout;     /* νģʽ½ճʱʱ(λms) */
    uint32_t tx_timeout;         /* ͳʱʹ [0:  1: ʹ] */
    uint8_t  crc_len;            /* ӲCRCУֽշ˫һ£ */
                                 /* [FSK_CRC_OFF         CRCܹر]
                                  * [FSK_CRC_1_BYTE      1ֽCRC]
                                  * [FSK_CRC_2_BYTE      2ֽCRC]
                                  * [FSK_CRC_1_BYTE_INV  1ֽCRC㲢ת]
                                  * [FSK_CRC_2_BYTE_INV  2ֽCRC㲢ת]
                                  * [FSK_CRC_2_BYTES_IBM IBM׼2ֽCRC]
                                  * [FSK_CRC_2_BYTES_CCITCCIT׼2ֽCRC]*/
    uint8_t  crc_software;       /* CRC[0: ر, 1: ]շ˫һ£ */
    uint8_t  header_mode;        /* ݰ[0ȹ̶1ȿɱ]շ˫һ£ */
    uint8_t  payload_len;        /* Чس */
} sx126x_fsk_param_set_t;

/**
 * \brief  ¼ص
 */
typedef struct sx126x_fsk_events {

    /**
     * \brief ɻصԭ
     */
    void (*pfn_tx_done_cb)(void);

    /**
     * \brief 䳬ʱصԭ
     */
    void (*pfn_tx_timeout_cb)(void);

    /**
     * \brief ɻصԭ
     *
     * \param[in] p_payload : ָջ
     * \param[in] size      : յݴС
     * \param[in] rssi      : ո֡ʱ RSSI ֵ [dBm]
     * \param[in] snr       : Ӳԭʼ
     *                         FSK : N/A (set to 0)
     *                         LoRa: SNR value in dB
     */
    void (*pfn_rx_done_cb)(uint8_t *p_payload, uint16_t size, int16_t rssi, int8_t snr);

    /**
     * \brief ճʱصԭ
     */
    void (*pfn_rx_timeout_cb)(void);

    /**
     * \brief ӲCRCصԭ
     */
    void (*pfn_rx_hw_crc_error_cb)(void);

    /**
     * \brief CRCصԭ
     */
    void (*pfn_rx_sw_crc_error_cb)(void);

} sx126x_fsk_events_t;


typedef struct radio_sx126x_fsk_devinfo {

   /** \brief ¼ص,ûҪΪNULL */
    sx126x_fsk_events_t    *p_events;

    /** \brief ò */
    sx126x_fsk_param_set_t *p_fsk_param;
} radio_sx126x_fsk_devinfo_t;


/**
 * \brief sx126x 豸ṹ
 */
typedef struct radio_sx126x_fsk_dev {
    radio_dev_t                 isa;
    sx126x_spi_funcs_t         *p_spi;
    sx126x_gpio_funcs_t        *p_gpio;
    sx126x_delay_funcs_t       *p_delay;
    sx126x_cpu_lock_funcs_t    *p_lock;
    sx126x_run_state_t          run_state;
    radio_sx126x_fsk_devinfo_t *p_devinfo;
} radio_sx126x_fsk_dev_t;



radio_handle_t radio_sx126x_fsk_init (radio_sx126x_fsk_dev_t      *p_sx126x,
                                      sx126x_spi_funcs_t          *p_spi,
                                      sx126x_gpio_funcs_t         *p_gpio,
                                      sx126x_delay_funcs_t        *p_delay,
                                      sx126x_cpu_lock_funcs_t     *p_cpu_lock,
                                      radio_sx126x_fsk_devinfo_t  *p_devinfo);




















#endif  //wjq

