/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SX126x LoRa driver
 *
 * \internal
 * \par Modification history
 * - 1.00 19-09-09  sup, first implementation.
 * \endinternal
 */


#ifndef SX126X_LORA_H_
#define SX126X_LORA_H_

#include "stdbool.h"
#include "radio.h"
#include "stdint.h"
#include "sx126x_common.h"



/**
 * \brief LoRaģʽؼĴ
 */
#define    REG_ADDR_LORA_SYNCWORD_MSB    0x0740  /* LoRaͬMSBĴ */
#define    REG_ADDR_LORA_SYNCWORD_LSB    0x0741  /* LoRaͬLSBĴ */



/**
 * \brief CADʱķ
 */
#define    LORA_CAD_01_SYMBOL    0x00
#define    LORA_CAD_02_SYMBOL    0x01
#define    LORA_CAD_04_SYMBOL    0x02
#define    LORA_CAD_08_SYMBOL    0x03
#define    LORA_CAD_16_SYMBOL    0x04



/**
 * \brief CAD֮Ϊ
 */
#define    LORA_CAD_ONLY    0x00
#define    LORA_CAD_RX      0x01
#define    LORA_CAD_LBT     0x10



/**
 * \brief CAD״̬
 */
#define    CAD_STATE_START     0x01
#define    CAD_STATE_FINISH    0x02
#define    CAD_STATE_DONE      0x04
#define    CAD_STATE_DETECT    0x08



/**
 * \brief LoRaģʽĴֵ
 */
#define    LORA_BW_500    6
#define    LORA_BW_250    5
#define    LORA_BW_125    4
#define    LORA_BW_062    3
#define    LORA_BW_041    10
#define    LORA_BW_031    2
#define    LORA_BW_020    9
#define    LORA_BW_015    1
#define    LORA_BW_010    8
#define    LORA_BW_007    0



/**
 * \brief LoRaģʽͷģʽ
 */
typedef enum
{
    LORA_PACKET_VARIABLE_LENGTH      = 0x00,         
    LORA_PACKET_FIXED_LENGTH         = 0x01,        
    LORA_PACKET_EXPLICIT             = LORA_PACKET_VARIABLE_LENGTH,
    LORA_PACKET_IMPLICIT             = LORA_PACKET_FIXED_LENGTH,
}radio_lora_packet_lengths_mode_t;



/**
 * \brief LoRa
 */
typedef struct sx126x_lora_param_set {
    rf_dev_t dev_type;               /* 豸ͺѡ */
    uint32_t frequency;              /* Ƶ    */
    int8_t   power;                  /* 书 */
    uint8_t  bandwidth;              /*  */
    uint8_t  sf;                     /* Ƶ */
    uint8_t  coderate;               /*  [1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8] */
    uint8_t  crc_on;                 /* ӲCRC [0: ر, 1: ] */
    uint8_t  header_mode;            /* ͷģʽ [0: ʽͷģʽ, 1: ʽͷģʽ] */
    uint8_t  rx_mode;                /* ģʽ [0: Continuous, 1 Single] */
    uint32_t rx_pkt_timeout;         /* νģʽ½ճʱʱ(λms)   */
    uint32_t tx_timeout;             /* ͳʱʹ */
    uint8_t  payload_len;            /* س */
    uint16_t preamble_len;           /* ǰ볤 */
    uint8_t  invert_iq;              /* IQת [0: , 1: ת] */
    uint8_t  public_net_enable;      /* ʹ[0ܣʹ˽1ʹܣʹù] */
    uint8_t  crc_software;           /* CRC[0: ر, 1: ] */
    uint8_t  ldro_auto;              /* Żʽѡ[0: ֶ, 1: Զ] */
    uint8_t  ldro_enable;            /* ֶƵŻĿ(ǰʽѡΪ"ֶ")[0: ر, 1: ] */
} sx126x_lora_param_set_t;

/**
 * \brief  ¼ص
 */
typedef struct sx126x_lora_events{

    /**
     * \brief ɻصԭ
     */
    void (*pfn_tx_done_cb)(void);

    /**
     * \brief 䳬ʱصԭ
     */
    void (*pfn_tx_timeout_cb)(void);

    /**
     * \brief ɻصԭ
     *
     * \param[in] p_payload : ָջ
     * \param[in] size      : յݴС
     * \param[in] rssi      : ո֡ʱ RSSI ֵ [dBm]
     * \param[in] snr       : Ӳԭʼ
     *                         FSK : N/A (set to 0)
     *                         LoRa: SNR value in dB
     */
    void (*pfn_rx_done_cb)(uint8_t *p_payload, uint16_t size, int16_t rssi, int8_t snr);

    /**
     * \brief ճʱصԭ
     */
    void (*pfn_rx_timeout_cb)(void);

    /**
     * \brief ӲCRCصԭ
     */
    void (*pfn_rx_hw_crc_error_cb)(void);
    
    /**
     * \brief CRCصԭ
     */
    void (*pfn_rx_sw_crc_error_cb)(void);

    /**
     * \brief ŵԾɻصԭ
     *
     * \param[in] channel_activity_detected : ǷԾ
     */
    void (*pfn_cad_done_cb)(bool channel_activity_detected);

} sx126x_lora_events_t;


typedef struct radio_sx126x_lora_devinfo {

   /** \brief ¼ص ,ûҪΪNULL */
    sx126x_lora_events_t    *p_events;

    /** \brief ò */
    sx126x_lora_param_set_t *p_lora_param;
} radio_sx126x_lora_devinfo_t;





/**
 * \brief sx126x 豸ṹ
 */
typedef struct radio_sx126x_lora_dev {
    radio_dev_t                  isa;
    sx126x_spi_funcs_t          *p_spi;
    sx126x_gpio_funcs_t         *p_gpio;
    sx126x_delay_funcs_t        *p_delay;
    sx126x_cpu_lock_funcs_t     *p_lock;
    sx126x_run_state_t           run_state;
    radio_sx126x_lora_devinfo_t *p_devinfo;
} radio_sx126x_lora_dev_t;





/**
 * \brief ʼ
 *
 * \param[in]  p_sx126x  : 豸
 * \param[in]  p_spi     : SPI
 * \param[in]  p_gpio    : GPIO
 * \param[in]  p_delay   : ʱ
 * \param[in]  p_irq     : жϺ
 * \param[in]  p_devinfo : 豸Ϣ
 *
 * \retval  NULL        : ʼʧ
 * \retval  ֵ       ʼɹ
 */
radio_handle_t radio_sx126x_lora_init (radio_sx126x_lora_dev_t     *p_sx126x,
                                       sx126x_spi_funcs_t          *p_spi,
                                       sx126x_gpio_funcs_t         *p_gpio,
                                       sx126x_delay_funcs_t        *p_delay,
                                       sx126x_cpu_lock_funcs_t     *p_cpu_lock,
                                       radio_sx126x_lora_devinfo_t *p_devinfo);



#endif



/* end of file */




