/*******************************************************************************
*                                 ZM4xx
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* ALL rights reserved.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* 
* The software is free to use in a commercial product and should not be
* redistributed in any way. See the LICENSE file for more details.
* 
* Contact information:
* web site:    http://www.zlg.cn/
*******************************************************************************/

/**
 * \file
 * \brief SX126x driver
 *
 * \internal
 * \par Modification history
 * - 1.00 22-08-15  sup, first implementation.
 * \endinternal
 */


#ifndef SX126X_COMMON_H_
#define SX126X_COMMON_H_


#include "stdint.h"


/**
 * \brief LoRaģʽFSKģʽͨüĴ
 */
#define    REG_RETENTION_LIST_BASE_ADDRESS          0x029F  /* Base address of the register retention list */
#define    REG_RAND_NUM_GEN                         0x0819  /* 32λɼĴ */
#define    REG_TX_MODULATION                        0x0889  /* Ϊ500kHzµĴ */
#define    REG_RX_GAIN                              0x08AC  /* ģʽüĴ */
#define    REG_TX_CLAMP_CONFIG                      0x08D8  /* ƥĴ */
#define    REG_OCP_CONFIG                           0x08E7  /* üĴ */
#define    REG_XTA_TRIM                             0x0911  /* XTAŵüĴ */
#define    REG_XTB_TRIM                             0x0912  /* XTBŵüĴ */
#define    REG_LR_PACKETPARAMS                      0x0704  /* ݰõļĴ */
#define    REG_LR_PAYLOADLENGTH                     0x0702  /* ЧسȵļĴ */
#define    REG_IQ_POLARITY                          0x0736  /* Optimizing the Inverted IQ Operation */

/**
 * @brief Maximum number of register that can be added to the retention list
 */
#define SX126X_MAX_NB_REG_IN_RETENTION 4



/**
 * \brief նյֽ
 */
#define    RX_MAX_PAYLOAD_LENTH    255



/**
 * \brief LoRaģͺѡ
 */
#define    RADIO_SX126X    0x01    /* SX126xϵLoRaģ */
#define    RADIO_LLCC68    0x02    /* LLCC68ϵLoRaģ */



/**
 * \brief LoRaģ龧ѡ
 */
#define    OSC_XTAL    0x01    /* Դ */
#define    OSC_TCXO    0x02    /* Դ */



/**
 * \brief LoRaģ鹤ģʽѡ
 */
#define STDBY_RC_MODE      0x00    /* RCģʽ     */
#define STDBY_XOSC_MODE    0x01    /* XOSCģʽ   */
#define RX_BOOST_MODE      0x11    /* ģʽ   */
#define RX_DC_MODE         0x22    /* ռձģʽ */
#define FS_MODE            0x33    /* мƵģʽ   */



/**
 * \brief 豸緽ʽѡ
 */
#define    USE_LDO     0x00
#define    USE_DCDC    0x01



/**
 * \brief ݰͣҲǵƷʽ
 */
#define    PACKET_TYPE_GFSK    0x00
#define    PACKET_TYPE_LORA    0x01
#define    PACKET_TYPE_NONE    0x0F



/**
 * \brief ʷŴбʱ䶨
 */
#define    RADIO_RAMP_10_US      0x00
#define    RADIO_RAMP_20_US      0x01
#define    RADIO_RAMP_40_US      0x02
#define    RADIO_RAMP_80_US      0x03
#define    RADIO_RAMP_200_US     0x04
#define    RADIO_RAMP_800_US     0x05
#define    RADIO_RAMP_1700_US    0x06
#define    RADIO_RAMP_3400_US    0x07



/**
 * \brief DIO3TCXOʱʹõĵѹ
 */
#define    TCXO_CTRL_1_6V    0x00
#define    TCXO_CTRL_1_7V    0x01
#define    TCXO_CTRL_1_8V    0x02
#define    TCXO_CTRL_2_2V    0x03
#define    TCXO_CTRL_2_4V    0x04
#define    TCXO_CTRL_2_7V    0x05
#define    TCXO_CTRL_3_0V    0x06
#define    TCXO_CTRL_3_3V    0x07


/**
 * \brief ߿жб
 */
#define    IRQ_RADIO_NONE               0x0000
#define    IRQ_TX_DONE                  0x0001
#define    IRQ_RX_DONE                  0x0002
#define    IRQ_PREAMBLE_DETECTED        0x0004
#define    IRQ_SYNCWORD_VALID           0x0008
#define    IRQ_HEADER_VALID             0x0010
#define    IRQ_HEADER_ERROR             0x0020
#define    IRQ_CRC_ERROR                0x0040
#define    IRQ_CAD_DONE                 0x0080
#define    IRQ_CAD_ACTIVITY_DETECTED    0x0100
#define    IRQ_RX_TX_TIMEOUT            0x0200
#define    IRQ_RADIO_ALL                0xFFFF


/**
 * \brief б
 */
typedef enum radio_commands_e
{
    RADIO_GET_STATUS                        = 0xC0,
    RADIO_WRITE_REGISTER                    = 0x0D,
    RADIO_READ_REGISTER                     = 0x1D,
    RADIO_WRITE_BUFFER                      = 0x0E,
    RADIO_READ_BUFFER                       = 0x1E,
    RADIO_SET_SLEEP                         = 0x84,
    RADIO_SET_STANDBY                       = 0x80,
    RADIO_SET_FS                            = 0xC1,
    RADIO_SET_TX                            = 0x83,
    RADIO_SET_RX                            = 0x82,
    RADIO_SET_RXDUTYCYCLE                   = 0x94,
    RADIO_SET_CAD                           = 0xC5,
    RADIO_SET_TXCONTINUOUSWAVE              = 0xD1,
    RADIO_SET_TXCONTINUOUSPREAMBLE          = 0xD2,
    RADIO_SET_PACKETTYPE                    = 0x8A,
    RADIO_GET_PACKETTYPE                    = 0x11,
    RADIO_SET_RFFREQUENCY                   = 0x86,
    RADIO_SET_TXPARAMS                      = 0x8E,
    RADIO_SET_PACONFIG                      = 0x95,
    RADIO_SET_CADPARAMS                     = 0x88,
    RADIO_SET_BUFFERBASEADDRESS             = 0x8F,
    RADIO_SET_MODULATIONPARAMS              = 0x8B,
    RADIO_SET_PACKETPARAMS                  = 0x8C,
    RADIO_GET_RXBUFFERSTATUS                = 0x13,
    RADIO_GET_PACKETSTATUS                  = 0x14,
    RADIO_GET_RSSIINST                      = 0x15,
    RADIO_GET_STATS                         = 0x10,
    RADIO_RESET_STATS                       = 0x00,
    RADIO_CFG_DIOIRQ                        = 0x08,
    RADIO_GET_IRQSTATUS                     = 0x12,
    RADIO_CLR_IRQSTATUS                     = 0x02,
    RADIO_CALIBRATE                         = 0x89,
    RADIO_CALIBRATEIMAGE                    = 0x98,
    RADIO_SET_REGULATORMODE                 = 0x96,
    RADIO_GET_ERROR                         = 0x17,
    RADIO_CLR_ERROR                         = 0x07,
    RADIO_SET_TCXOMODE                      = 0x97,
    RADIO_SET_TXFALLBACKMODE                = 0x93,
    RADIO_SET_RFSWITCHMODE                  = 0x9D,
    RADIO_SET_STOPRXTIMERONPREAMBLE         = 0x9F,
    RADIO_SET_LORASYMBTIMEOUT               = 0xA0,
}radio_commands_t;



/**
 * \brief ṩоƬʱƵʺƵʲֵ
 *
 * ЩڼƵõRFƵֵ
 */
#define    XTAL_FREQ    (double)32000000
#define    FREQ_DIV     (double)33554432
#define    FREQ_STEP    0.95367431640625



/**
 * \brief ģʽ
 */
#define    CONTINUOUS_RECV_MODE    0
#define    SINGLE_RECV_MODE        1



/**
 * \name ״̬
 */
typedef enum {
    RFLR_STATE_SLEEP,
    RFLR_STATE_STDBY_RC,
    RFLR_STATE_STDBY_XOSC,
    RFLR_STATE_FS,
    RFLR_STATE_IDLE,
    RFLR_STATE_RX_INIT,
    RFLR_STATE_RX_RUNNING,
    RFLR_STATE_RX_DC,
    RFLR_STATE_RX_DONE,
    RFLR_STATE_RX_TIMEOUT,
    RFLR_STATE_RX_ERROR,
    RFLR_STATE_TX_INIT,
    RFLR_STATE_TX_RUNNING,
    RFLR_STATE_TX_DONE,
    RFLR_STATE_TX_TIMEOUT,
    RFLR_STATE_CAD_INIT,
    RFLR_STATE_CAD_RUNNING,
} sx126x_sm_t;



/**
 * \brief ״̬
 *  cad_stateֵοgrp_lora_cad_state
 *  sm_state ֵοgrp_lora_state
 */
typedef struct sx126x_setting {
    uint8_t      cad_state;
    sx126x_sm_t  sm_state;
} sx126x_run_state_t;



/**
 * \brief ֵ
 */
enum {
    LORA_RET_CODE_OK = 0,
    LORA_RET_CODE_ERR,
    LORA_RET_CODE_NO_CAD_DONE_IRQ,
    LORA_RET_CODE_NO_CAD_DETECT_IRQ,
    LORA_RET_CAD_RUNNING,
    LORA_RET_RECV_CRC_ERR
};



/**
 * \brief 豸ͺѡ
 */
typedef struct rf_dev{
    uint8_t   rf_type;     /* ģͺѡ */
    uint8_t   osc_type;    /* ѡ        */
    uint8_t   cap_val;     /* XTAXTBŵֵ */
    uint32_t  min_freq;    /* ֵ֧СƵ */
    uint32_t  max_freq;    /* ֵ֧Ƶ */
} rf_dev_t;



/**
 * \brief SPI
 * \note: úƬѡ
 */
typedef struct sx126x_spi_funcs {
    /** \brief SPI һֽںýӿڱṩ */
    void    (*pfn_spi_write_byte) (uint8_t byte);

    /** \brief SPI һֽںýӿڱṩ */
    uint8_t (*pfn_spi_read_byte) (void);
} sx126x_spi_funcs_t;


/**
 * \brief GPIO 
 */
typedef struct sx126x_gpio_funcs {
    
    /*
     * SX_NRESET λŵƽáýӿڱṩ
     * val=0ʾΪ͵ƽval = 1ʾΪߵƽ
     */  
    void    (*pfn_reset_pin_set) (uint8_t val);

    /*
     * SPI_NSS Ƭѡŵƽáýӿڱṩ
     * val=0ʾΪ͵ƽval = 1ʾΪߵƽ
     */ 
    void    (*pfn_sel_pin_set)   (uint8_t val);

    /*
     * Ƶطŵƽáýӿڿɸģѡṩ
     * ΪZSL420ZSL421ZM1268S ZM68Sϵģ飬ṩ˽ӿ
     * val=0ʾΪ͵ƽval = 1ʾΪߵƽ
     * ΪZSL64ϵģ飬ֱô˽ӿΪNULL߽ӿʵΪ
     */
    void    (*pfn_txen_pin_set)   (uint8_t val);

    /*
     * VCTL_1(RXEN) Ƶؽտŵƽáýӿڱṩ
     * val=0ʾΪ͵ƽval = 1ʾΪߵƽ
     */
    void    (*pfn_rxen_pin_set)   (uint8_t val);

    /*
     * BUSY æŵƽȡýӿڱṩ
     * 0ʾΪ͵ƽ1ʾΪߵƽ
     */
    uint8_t (*pfn_busy_pin_read)  (void);

} sx126x_gpio_funcs_t;

/**
 * \brief ʱ
 */
typedef struct sx126x_delay_funcs {
    /** \brief ʱýӿڱṩ */
    void (*pfn_delay_ms) (uint32_t ms);
    
    /** \brief ΢ʱ */
    void (*pfn_delay_us) (uint32_t us);
} sx126x_delay_funcs_t;


/**
 * \brief CPU
 * ĺڱSPI䣬ܻᵼģ
 */
typedef struct sx126x_cpu_lock_funcs {
    /** \brief CPUжϡýӿڱṩ */
    uint32_t (*pfn_cpu_lock) (void);

    /** \brief CPUжϡýӿڱṩ */
    void (*pfn_cpu_unlock) (uint32_t cpu_key);
} sx126x_cpu_lock_funcs_t;



#endif 
